package test.testcases;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import model.ss.Queue;
import test.testutil.Car;
import test.testutil.Employee;

public class QueueTestCase extends TestCase {

	//=======================================
	//Smoke Tests - Construction

	public void test_empty() {
		Queue<Employee> s = new Queue<>();
		assertEquals(true, s.isEmpty(), "isEmpty");
	}

	//=======================================
	//Main	

	public void test_enqueue_dequeue_3(boolean linkedType) {
		//If linkedType is true, then structure engine uses LinkedList,
		//otherwise it is an arrayed type (used DynamicArray)
		Car[] cars = Car.newFour();
		Queue<Car> s = new Queue<>();
		if (linkedType) 
			s.setLinked();
		else
			s.setArrayed();		
		//queue up all cars
		for (Car nextCar : cars)
			s.enqueue(nextCar);
		//Release all cars making sure they come out in expected first-in first-out order
		for (int i = 0; i < cars.length; i++) {
			int actualId = s.dequeue().getId(), expectedId = cars[i].getId();
			this.assertEquals(actualId, expectedId, "test_enqueue_dequeue_3");
		}
	}

	public void test_traffic_simulation(boolean linkedType) {
		//If linkedType is true, then structure engine uses LinkedList,
		//otherwise it is an arrayed type (used DynamicArray)
		Queue<Car> trafficLightQueue = new Queue<>();
		if (linkedType) 
			trafficLightQueue.setLinked();
		else
			trafficLightQueue.setArrayed();
		//200 time units each unit adds 5 to queue and releases 3 from queue
		//so in end we should have bad traffic jam at traffic light, queue
		//should contains 200 * 2 = 400 cars
		for (int timeUnit = 1, id = 1001; timeUnit <= 200; timeUnit++) {
			//Add five to queue
			for (int j = 1; j <= 5; j++, id++) {
				Car nextCar = Car.fromIdName(id, "Name-"+id);
				trafficLightQueue.enqueue(nextCar);
			}
			//Release three from queue
			for (int k = 1; k <= 3; k++)
				trafficLightQueue.dequeue();
		}
		this.assertEquals(400, trafficLightQueue.size(), "test_traffic_simulation");
	}
				
		
	//=======================================
	//Main

	public static void main(String[] args) {
		QueueTestCase tc = new QueueTestCase();
		tc.test_empty();
		tc.test_enqueue_dequeue_3(true); //linked		
		tc.test_enqueue_dequeue_3(false); //arrayed		
		tc.test_traffic_simulation(true);
		tc.test_traffic_simulation(false);

		tc.printSummary();
	}

}