package test.testcases;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import model.ss.Stack;
import test.testutil.Employee;

public class StackTestCase extends TestCase {
	
	//=======================================
	//Smoke Tests - Construction

	public void test_empty() {
		Stack<Employee> s = new Stack<>();
		assertEquals(true, s.isEmpty(), "isEmpty");
	}
	
	//=======================================
	//Main	
	
	public void test_push_pop_3(boolean linkedType) {
		//If linkedType is true, then structure engine uses LinkedList,
		//otherwise it is an arrayed type (used DynamicArray)
		int 
			sz = 3,
			startId = 1001;
		Stack<Employee> s = new Stack<>();
		if (linkedType) 
			s.setLinked();
		else
			s.setArrayed();
		for (int id = startId; id < startId + sz; id++) {
			Employee emp = Employee.fromIdName(id, "name"+id);
			s.push(emp);
		}
		int[] expectedIds = new int[] {startId + sz - 1, startId + sz - 2, startId};
		//Pop and check
		//pop 1003, 1002, 1001
		for (int index = 0; index < sz; index++) {
			int
				actualId = s.pop().getId(),
				expectedId = expectedIds[index];
			this.assertEquals(expectedId, actualId, "test_push_pop_3");
		}
	}		
	
	public void test_push_pop_1000(boolean linkedType) {
		//If linkedType is true, then structure engine uses LinkedList,
		//otherwise it is an arrayed type (used DynamicArray)
		Stack<Employee> s = new Stack<>();
		if (linkedType) 
			s.setLinked();
		else
			s.setArrayed();
		for (int id=1001; id<=2000; id++) {
			Employee emp = Employee.fromIdName(id, "name"+id);
			s.push(emp);
		}
		//pop em all
		List<Employee> actualEmps = collectToEnd(s, (struc) -> ((Stack<Employee>)struc).pop());
		List<Integer> actualIds = actualEmps.stream().map(emp -> emp.getId()).collect(Collectors.toList());
		List<Integer> expectedIds = range(1001, 2001);
		//remember stack pops of 2000 (last in) first, so we need to reverse list
		Collections.reverse(expectedIds);
		this.assertEquals(expectedIds, actualIds, "push-pop compare all");
	}	

	//=======================================
	//Main

	public static void main(String[] args) {
		StackTestCase tc = new StackTestCase();
		tc.test_empty();
		tc.test_push_pop_3(true);	//linked		
		tc.test_push_pop_3(false);	//arrayed		
		tc.test_push_pop_1000(true);
		tc.test_push_pop_1000(false);		
		tc.printSummary();
	}

}