package test.testcases;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import model.ss.SpecializedStructure;

public class TestCase {

	private int testCount;
	private int failed;
	private List<String> messages;

	//=======================================
	// Constructors

	public TestCase() {
		super();
		this.messages = new ArrayList<>();
	}

	//=======================================
	// Comparing	

	protected void assertEquals(Object expected, Object actual, String msg) {
		++testCount;
		boolean equals = expected.equals(actual);
		if (!equals) {
			++failed;
			messages.add(String.format("Failed: %s -- %s", msg, comparisonMsg(expected, actual)));
		}
	}
	
	protected String comparisonMsg(Object expected, Object actual) {
		if (List.class.isAssignableFrom(expected.getClass())) {
			List<?> 
				e = (List<?>)expected,
				a = (List<?>)actual;
				if (e.size() == a.size())
					return findUnmatched(e, a);
				return String.format("expected size vs actual size = [%d] vs [%d]", e.size(), a.size(), null);
		}
		return String.format("expected vs actual= [%s] vs [%s]", expected.toString(), actual.toString());
	}
				
	private String findUnmatched(List<?> e, List<?> a) {
		//Assumption: size of lists are equal
		for (int i = 0; i < e.size(); i++) {
			Object 
				e1 = e.get(i),
				a1 = a.get(i);
			if (!e1.equals(a1)) {
				return String.format("Unmatched elements at index %d -- [%s] vs [%s]",
						i, e1.toString(), a1.toString());
			}
		}
		return "";
	}

	//=======================================
	// Summary

	protected void printSummary() {
		int count = this.testCount;
		int passed = count - this.failed;
		printf("Test Count: %d%n", count);
		printf("Passed Count: %d%n", passed);
		printf("Passed Percent %.1f%%%n", 100.0 * (passed / (double) count));
		if (passed < count) {
			prn("Error Messages:");
			for (String msg: this.messages)
				printf("\t%s%n", msg);
		}

	}

	//=======================================
	// Static Helpers

	public static void prn(Object o) {
		System.out.println(o);
	}

	public static void printf(String template, Object... args) {
		System.out.printf(template, args);
	}

	public static <T> List<T> collectToEnd(SpecializedStructure<T> structure,
			Function<SpecializedStructure<T>, T> getFct) {
		List<T> newList = new ArrayList<>();
		while (!structure.isEmpty())
			newList.add(getFct.apply(structure));
		return newList;
	}
	
	public static List<Integer> range(int start, int stop) {
		//inclusive, exclusive
		return
			IntStream.range(start, stop)
				.boxed()
				.collect(Collectors.toList());
	}
	
}
